@@ -3,5 +3,7 @@
 
 void showxml(void);
 void xmldate(time_t *date, int type);
+void xmlheader(void);
+void xmlfooter(void);
 
 #endif
