@@ -1,5 +1,5 @@
 /*
-vnStat - Copyright (c) 2002-2014 Teemu Toivola <tst@iki.fi>
+vnStat - Copyright (c) 2002-2015 Teemu Toivola <tst@iki.fi>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,18 +19,20 @@ vnStat - Copyright (c) 2002-2014 Teemu Toivola <tst@iki.fi>
 #include "ifinfo.h"
 #include "traffic.h"
 #include "dbxml.h"
+#include "dbjson.h"
 #include "dbshow.h"
 #include "dbaccess.h"
 #include "dbmerge.h"
 #include "misc.h"
 #include "cfg.h"
+#include "ibw.h"
 #include "vnstat.h"
 
 int main(int argc, char *argv[]) {
 
 	int i, currentarg;
-	DIR *dir=NULL;
-	struct dirent *di=NULL;
+	DIR *dir = NULL;
+	struct dirent *di = NULL;
 	PARAMS p;
 
 	initparams(&p);
@@ -59,6 +61,9 @@ int main(int argc, char *argv[]) {
 	if (!loadcfg(p.cfgfile)) {
 		return 1;
 	}
+	if (!ibwloadcfg(p.cfgfile)) {
+		return 1;
+	}
 
 	if (cfg.locale[0]!='-' && strlen(cfg.locale)>0) {
 		setlocale(LC_ALL, cfg.locale);
@@ -206,6 +211,8 @@ int main(int argc, char *argv[]) {
 			cfg.qmode=9;
 		} else if (strcmp(argv[currentarg],"--xml")==0) {
 			cfg.qmode=8;
+		} else if (strcmp(argv[currentarg],"--json")==0) {
+			cfg.qmode=10;
 		} else if (strcmp(argv[currentarg],"--savemerged")==0) {
 			p.savemerged=1;
 		} else if ((strcmp(argv[currentarg],"-ru")==0) || (strcmp(argv[currentarg],"--rateunit"))==0) {
@@ -275,7 +282,7 @@ int main(int argc, char *argv[]) {
 			p.delete=1;
 			p.query=0;
 		} else if (strcmp(argv[currentarg],"--iflist")==0) {
-			getiflist(&p.ifacelist);
+			getiflist(&p.ifacelist, 1);
 			printf("Available interfaces: %s\n", p.ifacelist);
 			free(p.ifacelist);
 			return 0;
@@ -351,7 +358,7 @@ int main(int argc, char *argv[]) {
 
 		/* give more help if there's no database */
 		if (p.files==0) {
-			getiflist(&p.ifacelist);
+			getiflist(&p.ifacelist, 1);
 			printf("No database found, nothing to do. Use --help for help.\n\n");
 			printf("A new database can be created with the following command:\n");
 			printf("    %s --create -i eth0\n\n", argv[0]);
@@ -451,6 +458,7 @@ void showlonghelp(PARAMS *p)
 			printf("         --oneline             show simple parseable format\n");
 			printf("         --exportdb            dump database in text format\n");
 			printf("         --importdb            import previously exported database\n");
+			printf("         --json                show database in json format\n");
 			printf("         --xml                 show database in xml format\n");
 
 			printf("   Modify:\n");
@@ -689,7 +697,7 @@ void handlecreate(PARAMS *p)
 	}
 
 	if (!getifinfo(p->interface) && !p->force) {
-		getiflist(&p->ifacelist);
+		getiflist(&p->ifacelist, 1);
 		printf("Only available interfaces can be added for monitoring.\n\n");
 		printf("The following interfaces are currently available:\n    %s\n", p->ifacelist);
 		free(p->ifacelist);
@@ -717,8 +725,8 @@ void handlecreate(PARAMS *p)
 
 void handleupdate(PARAMS *p)
 {
-	DIR *dir=NULL;
-	struct dirent *di=NULL;
+	DIR *dir = NULL;
+	struct dirent *di = NULL;
 	time_t current;
 
 	if (!p->update) {
@@ -804,7 +812,7 @@ void handleupdate(PARAMS *p)
 		}
 
 		if (!getifinfo(data.interface) && !p->force) {
-			getiflist(&p->ifacelist);
+			getiflist(&p->ifacelist, 1);
 			printf("Only available interfaces can be added for monitoring.\n\n");
 			printf("The following interfaces are currently available:\n    %s\n", p->ifacelist);
 			free(p->ifacelist);
@@ -834,8 +842,9 @@ void handleupdate(PARAMS *p)
 
 void handleshowdatabases(PARAMS *p)
 {
-	DIR *dir=NULL;
-	struct dirent *di=NULL;
+	DIR *dir = NULL;
+	struct dirent *di = NULL;
+	int dbcount = 0;
 
 	if (!p->query) {
 		return;
@@ -847,7 +856,7 @@ void handleshowdatabases(PARAMS *p)
 		if (p->files==0) {
 			/* printf("No database found.\n"); */
 			p->query=0;
-		} else if ((cfg.qmode==0 || cfg.qmode==8) && (p->files>1)) {
+		} else if ((cfg.qmode==0 || cfg.qmode==8 || cfg.qmode==10) && (p->files>1)) {
 
 			if (cfg.qmode==0) {
 				if (cfg.ostyle!=0) {
@@ -855,8 +864,10 @@ void handleshowdatabases(PARAMS *p)
 				} else {
 					printf("\n                      rx      /      tx      /     total\n");
 				}
-			} else {
-				printf("<vnstat version=\"%s\" xmlversion=\"%d\">\n", VNSTATVERSION, XMLVERSION);
+			} else if (cfg.qmode==8) {
+				xmlheader();
+			} else if (cfg.qmode==10) {
+				jsonheader();
 			}
 			if ((dir=opendir(p->dirname))==NULL) {
 				return;
@@ -872,14 +883,19 @@ void handleshowdatabases(PARAMS *p)
 				if (!p->newdb) {
 					if (cfg.qmode==0) {
 						showdb(5);
-					} else {
+					} else if (cfg.qmode==8) {
 						showxml();
+					} else if (cfg.qmode==10) {
+						showjson(dbcount);
 					}
+					dbcount++;
 				}
 			}
 			closedir(dir);
 			if (cfg.qmode==8) {
-				printf("</vnstat>\n");
+				xmlfooter();
+			} else if (cfg.qmode==10) {
+				jsonfooter();
 			}
 
 		/* show in qmode if there's only one file or qmode!=0 */
@@ -895,12 +911,16 @@ void handleshowdatabases(PARAMS *p)
 						printf("\n                      rx      /      tx      /     total\n");
 					}
 				}
-				if (cfg.qmode!=8) {
+				if (cfg.qmode!=8 && cfg.qmode!=10) {
 					showdb(cfg.qmode);
-				} else {
-					printf("<vnstat version=\"%s\" xmlversion=\"%d\">\n", VNSTATVERSION, XMLVERSION);
+				} else if (cfg.qmode==8) {
+					xmlheader();
 					showxml();
-					printf("</vnstat>\n");
+					xmlfooter();
+				} else if (cfg.qmode==10) {
+					jsonheader();
+					showjson(dbcount);
+					jsonfooter();
 				}
 			}
 		}
@@ -918,12 +938,16 @@ void handleshowdatabases(PARAMS *p)
 					printf("\n                      rx      /      tx      /     total\n");
 				}
 			}
-			if (cfg.qmode!=8) {
+			if (cfg.qmode!=8 && cfg.qmode!=10) {
 				showdb(cfg.qmode);
-			} else {
-				printf("<vnstat version=\"%s\" xmlversion=\"%d\">\n", VNSTATVERSION, XMLVERSION);
+			} else if (cfg.qmode==8) {
+				xmlheader();
 				showxml();
-				printf("</vnstat>\n");
+				xmlfooter();
+			} else if (cfg.qmode==10) {
+				jsonheader();
+				showjson(dbcount);
+				jsonfooter();
 			}
 		}
 	}
