@@ -6,6 +6,7 @@
 #include "dbcache.h"
 #include "dbshow.h"
 #include "cfg.h"
+#include "ibw.h"
 
 START_TEST(initdb_activates_database)
 {
@@ -732,7 +733,7 @@ START_TEST(dbcheck_with_no_changes_in_iflist)
 	ck_assert_int_eq(remove_directory(TESTDIR), 1);
 	fake_proc_net_dev("w", "ethsomething", 1, 2, 3, 4);
 	fake_proc_net_dev("a", "ethelse", 5, 6, 7, 8);
-	ck_assert_int_ne(getiflist(&ifacelist), 0);
+	ck_assert_int_ne(getiflist(&ifacelist, 0), 0);
 	ifhash = simplehash(ifacelist, (int)strlen(ifacelist));
 
 	ck_assert_int_eq(dbcheck(ifhash, &forcesave), ifhash);
@@ -937,7 +938,6 @@ END_TEST
 
 void add_database_tests(Suite *s)
 {
-	/* Database test cases */
 	TCase *tc_db = tcase_create("Database");
 	tcase_add_test(tc_db, initdb_activates_database);
 	tcase_add_test(tc_db, cleanhours_really_cleans_hours);
